"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComposerFactory = void 0;
var os_1 = require("os");
var composer_1 = require("./targets/Microsoft.VisualStudio.Services/composer");
var composer_2 = require("./targets/Microsoft.VisualStudio.Services.Integration/composer");
var composer_3 = require("./targets/Microsoft.VisualStudio.Offer/composer");
var _ = require("lodash");
var trace = require("../../../lib/trace");
var ComposerFactory = /** @class */ (function () {
    function ComposerFactory() {
    }
    ComposerFactory.GetComposer = function (settings, targets) {
        var composers = [];
        // @TODO: Targets should be declared by the composer
        targets.forEach(function (target) {
            switch (target.id) {
                case "Microsoft.VisualStudio.Services":
                case "Microsoft.VisualStudio.Services.Cloud":
                case "Microsoft.TeamFoundation.Server":
                    composers.push(new composer_1.VSSExtensionComposer(settings));
                    break;
                case "Microsoft.VisualStudio.Services.Integration":
                case "Microsoft.TeamFoundation.Server.Integration":
                case "Microsoft.VisualStudio.Services.Cloud.Integration":
                    composers.push(new composer_2.VSSIntegrationComposer(settings));
                    break;
                case "Microsoft.VisualStudio.Offer":
                    composers.push(new composer_3.VSOfferComposer(settings));
                    break;
                default:
                    if (!settings.bypassValidation) {
                        throw new Error("'" +
                            target.id +
                            "' is not a recognized target. Valid targets are 'Microsoft.VisualStudio.Services', 'Microsoft.VisualStudio.Services.Integration', 'Microsoft.VisualStudio.Offer'");
                    }
                    break;
            }
        });
        if (composers.length === 0 && targets.length === 0) {
            trace.warn("No recognized targets found. Ensure that your manifest includes a target property. E.g. \"targets\":[{\"id\":\"Microsoft.VisualStudio.Services\"}],..." + os_1.EOL + "Defaulting to Microsoft.VisualStudio.Services");
            composers.push(new composer_1.VSSExtensionComposer(settings));
        }
        // Build a new type of composer on the fly that is the
        // concatenation of all of the composers necessary for
        // this extension.
        var PolyComposer = (function () {
            function PolyComposer(settings) {
                this.settings = settings;
            }
            PolyComposer.prototype.getBuilders = function () {
                return _.uniqWith(composers.reduce(function (p, c) {
                    return p.concat(c.getBuilders());
                }, []), function (b1, b2) { return b1.getType() === b2.getType(); });
            };
            PolyComposer.prototype.validate = function (components) {
                return Promise.all(composers.reduce(function (p, c) {
                    return p.concat(c.validate(components));
                }, [])).then(function (multi) {
                    // flatten
                    return multi.reduce(function (p, c) { return p.concat(c); }, []);
                });
            };
            return PolyComposer;
        })();
        return new PolyComposer(settings);
    };
    return ComposerFactory;
}());
exports.ComposerFactory = ComposerFactory;
//# sourceMappingURL=extension-composer-factory.js.map